;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.1 - GPL3 - Feb 2012
;
;-----------------------------------------------------------------------------------
; 
; RM_BiosE820 - Call BIOS INT 15h AX=0E820h
;
; Set memory mapping
;
; Based on osdev wiki
;-----------------------------------------------------------------------------------

USE16

mem_table		dw 4000h
msg_NotSupported        db 'E820 Function is not supported.',13,10,0
mmap_ent		dw 0
;-------------------------------------------------------------------

; Get the BIOS E820 Memory Map
; use the INT 0x15, eax= 0xE820 BIOS function to get a memory map
; inputs: es:di -> destination for 24 byte entries
RM_BiosE820:
    	pusha

; Clear out 4096 bytes of memory.
	mov ecx, 1024
	xor eax, eax
	mov  di, [mem_table]
	rep stosd

.do_e820:
	mov  di, [mem_table]
	xor ebx, ebx			; ebx must be 0 to start
	xor bp, bp			; keep an entry count in bp (for debugging)
	mov edx, 0x0534D4150		; Place "SMAP" into edx
	mov eax, 0xe820
	mov [es:di + 20], dword 1	; force a valid ACPI 3.X entry
	mov ecx, 24			; ask for 24 bytes
	int 0x15
	jc short .failed		; carry set on first call means "unsupported function"
	mov edx, 0x0534D4150		; Some BIOSes apparently trash this register?
	cmp eax, edx			; on success, eax must have been reset to "SMAP"
	jne short .failed
	test ebx, ebx			; ebx = 0 implies list is only 1 entry long (worthless)
	je short .failed
	jmp short .jmpin
.e820lp:
	mov eax, 0xe820			; eax, ecx get trashed on every int 0x15 call
	mov [es:di + 20], dword 1	; force a valid ACPI 3.X entry
	mov ecx, 24			; ask for 24 bytes again
	int 0x15
	jc short .e820f			; carry set means "end of list already reached"
	mov edx, 0x0534D4150		; repair potentially trashed register
.jmpin:
	jcxz .skipent			; skip any 0 length entries
	cmp cl, 20			; got a 24 byte ACPI 3.X response?
	jbe short .notext
	test byte [es:di + 20], 1	; if so: is the "ignore this data" bit clear?
	je short .skipent
.notext:
	mov ecx, [es:di + 8]		; get lower dword of memory region length
	or ecx, [es:di + 12]		; "or" it with upper dword to test for zero
	jz .skipent			; if length qword is 0, skip entry
	inc bp				; got a good entry: ++count, move to next storage spot
	add di, 32			; room in InfoTable

.skipent:
	test ebx, ebx			; if ebx resets to 0, list is complete
	jne short .e820lp
.e820f:
	mov [mmap_ent], bp		; store the entry count
	clc				; there is "jc" on end of list to this point, so the carry must be cleared
	jmp .end			; OK

.failed:
    	mov si, msg_NotSupported
    	call RM_PrintString
.StopE820:
	hlt				; halt cpu
	jmp $

.end:
    	popa
    	ret

; ===================================================================================================
; EOF

;------------------------------------------------------------------------------------------------
; BIOS Function: INT 0x15, EAX = 0xE820   ( from osdev wiki )
;
; By far the best way to detect the memory of a PC is by using the INT 0x15, EAX = 0xE820 command.
; This function is available on all PCs built since 2002, and on most existing PCs before then.
; It is the only BIOS function that can detect memory areas above 4G.
; It is meant to be the ultimate memory detection BIOS function.
;
; In reality, this function returns an unsorted list that may contain unused entries and
; (in rare/dodgy cases) may return overlapping areas.
; Each list entry is stored in memory at ES:DI, and DI is not incremented for you.
; The format of an entry is 2 qwords and a dword in the 20 byte version, plus one additional dword
; in the 24 byte ACPI 3.0 version (but nobody has ever seen a 24 byte one).
; It is probably best to always store the list entries as 24 byte quantities -- to preserve qword
; alignments, if nothing else. (Make sure to set that last dword to 1 before each call, to make
; your map compatible with ACPI).
;
;    * First qword  = Base address
;    * Second qword = Length of "region" (if this value is 0, ignore the entry)
;    * Next dword   = Region "type" 
;    * Next dword   = ACPI 3.0 Extended Attributes bitfield (if 24 bytes are returned, instead of 20) 
;          . Bit 0 of the Extended Attributes indicates if the entire entry should be ignored (if the bit is clear).
;	     This is going to be a huge compatibility problem because most current OSs won't read
;	     this bit and won't ignore the entry.
;          . Bit 1 of the Extended Attributes indicates if the entry is non-volatile (if the bit is set) or not.
;	     The standard states that "Memory reported as non-volatile may require characterization to determine
;	     its suitability for use as conventional RAM."
;          . The remaining 30 bits of the Extended Attributes are currently undefined. 
;
; Basic Usage:
; For the first call to the function, point ES:DI at the destination buffer for the list.
; Clear EBX. Set EDX to the magic number 0x534D4150. Set EAX to 0xE820 (note that the upper word
; of EAX should be set to 0). Set ECX to 24. Do an INT 0x15.
;
; If the first call to the function is successful, EAX will be set to 0x534D4150, and the Carry flag
; will be clear. EBX will be set to some non-zero value, which must be preserved for the next call
; to the function. CL will contain the number of bytes actually stored at ES:DI (probably 20).

;
; For the subsequent calls to the function: increment DI by your list entry size, reset EAX to 0xE820,
; and ECX to 24. When you reach the end of the list, EBX may reset to 0.
; If you call the function again with EBX = 0, the list will start over. If EBX does not reset to 0,
; the function will return with Carry set when you try to access the entry after the last valid entry. 


