;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.1 - GPL3 - Feb 2012
;
;-----------------------------------------------------------------------------------
; Enable CR registers
;-----------------------------------------------------------------------------------

PM_Init_CR:

	; Enable physical-address extensions (set CR4.PAE=1)
	mov	eax, cr4
	or	eax, 1 shl 5				; PAE (Bit 5)
	mov	cr4, eax 				

	; Point CR3 at PML4
	mov eax, PML4 + 00001000b			; Write-through (Bit 3)
	mov cr3, eax

	; Enable long mode (set EFER.LME=1)
	mov     ecx, 0xC0000080				; EFER MSR number
	rdmsr						; Read EFER
	or	eax,1 shl 8 				; LME (Bit 8) enable long mode
	wrmsr						; Write EFER

	; Enable paging to activate long mode (set CR0.PG=1)
	mov	eax, cr0
	or	eax, 1 shl 31				; PG (Bit 31)
	mov	cr0, eax 				; enable paging

ret

; ===================================================================================================
; EOF



; Control Registers
;--------------------------------------
;
; CR0
;-----
; 
;  63 62 61 60 59 58 57 56 55 54 53 52 51 50 49 48 47 46 45 44 43 42 41 40 39 38 37 36 35 34 33 32
;  -----------------------------------------------------------------------------------------------
; |                                                                                               |
; |                                     Reserved (must be zero)                                   | +4
; |                                                                                               |
;  -----------------------------------------------------------------------------------------------
; |P |C |N |                             |A |  |W |                             |N |E |T |E |M |P |
; |G |D |W |             R               |M |R |P |              R              |E |T |S |M |P |E | +0
; |  |  |  |                             |  |  |  |                             |  |  |  |  |  |  |
;  -----------------------------------------------------------------------------------------------
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;
; Protected-Mode Enable (PE) Bit. Bit 0.
; PE bit set to 1 = enable protected mode.
; PE bit cleared to 0 = disable protected mode.
; (in protected mode segment-protection mechanisms are enabled).
;
; Monitor Coprocessor (MP) Bit. Bit 1.
; If CR0.MP=1 and CR0.TS=1 => WAIT/FWAIT instruction causes a device-not-available exception.
; If CR0.MP=0 or CR0.TS=0 => WAIT/FWAIT instruction proceeds normally.
; Set MP to 1 if supports x87 instructions. This
;
; Emulate Coprocessor (EM) Bit. Bit 2.
; If CR0.EM=1 all x87 instructions cause a device-not-available exception.
; If CR0.EM=1 any of the 64-bit or 128-bit media instructions force an invalid-opcode exception.
; If CR0.EM=1 when the WAIT/FWAIT instruction is executed does not cause an device-not-available exception.
;
; Task Switched (TS) Bit. Bit 3.
; If CR0.TS=1 an x87 or media instruction cause a device-not-available exception.
; When a hardware task switch occurs CR0.TS is automatically set to 1.
; In long mode the hardware task-switch mechanism is not supported.
; (software task-switching can use the TS bit to control x87 and media instruction-unit context saves).
;
; Extension Type (ET) Bit. Bit 4, read-only.
; This bit is reserved and forced to 1 by the processor.
; (In early x86 processors = support of the 387DX math-coprocessor instruction set).
;
; Numeric Error (NE) Bit. Bit 5.
; NE bit cleared to 0 => disables internal control of x87 floating-point exceptions and enables external control.
; (When IGNNE# = 1 => x87 floating-point exceptions are ignored).
; (When IGNNE# = 0 => x87 floating-point exceptions are reported).
; NE bit set to 1 => enables internal control of x87 floating-point exceptions and disables external control.
; It is recommended that software set NE to 1.
;
; Write Protect (WP) Bit. Bit 16.
; WP set to 1 => supervisor software can not write into read-only pages.
; WP cleared to 0 => supervisor software can write into read-only pages.
;
; Alignment Mask (AM) Bit. Bit 18.
; CR0.AM=1 and eFLAGS.AC=1 => automatic alignment checking. (with CPL=3 a memory reference to
; an unaligned operand causes an alignment-check exception).
; CR0.AM=0 or eFLAGS.AC=0 => disable automatic alignment checking.
;
; Not Writethrough (NW) Bit. Bit 29.
; Ignored.
; (The NW bit exists only for legacy purposes).
;
; Cache Disable (CD) Bit. Bit 30.
; CD is cleared to 0 => the internal caches are enabled.
; CD is set to 1 => no new data or instructions into the internal caches.
; (However with CD=1 access the internal caches when read/write hit cache).
; CD=1 => ignore the page-level cache-control bits (PWT and PCD) when paging is enabled.
; Cache misses do not affect the internal caches when CD=1.
; (For preventing cache access : writing back and invalidating the caches before setting CD=1).
;
; Paging Enable (PG) Bit. Bit 31.
; PG=1 => enables page translation. (CR0.PE=1 must be set before PG=1)
; PG=0 => disables page translation.
; (CR0.PE=0 and PG=1 causes a general-protection exception)
;
; Reserved Bits. Bits 28-19, 17, 15-6, and 63-32.



; CR3 (PAE Paging Legacy-Mode)
;-----------------------------
;
;  -----------------------------------------------------------------------------------------------
; |                                                                                |P |P |        |
; |                      Page-Directory-Table Base Address                         |C |W |  Res.  | +0
; |                                                                                |D |T |        |
;  -----------------------------------------------------------------------------------------------
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;
;
; CR3 (Non-PAE Paging Legacy-Mode)
;-----------------------------
;
;  -----------------------------------------------------------------------------------------------
; |                                                           |                    |P |P |        |
; |           Page-Directory-Table Base Address               |      Reserved      |C |W |  Res.  | +0
; |                                                           |                    |D |T |        |
;  -----------------------------------------------------------------------------------------------
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;
;
; CR3 (Long Mode)
;----------------
; 
;  63 62 61 60 59 58 57 56 55 54 53 52 51 50 49 48 47 46 45 44 43 42 41 40 39 38 37 36 35 34 33 32
;  -----------------------------------------------------------------------------------------------
; |                                   |                                                           |
; |            Reserved               |          Page-Map Level-4 Table Base Address              | +4
; |                                   |                                                           |
;  -----------------------------------------------------------------------------------------------
; |                                                           |                    |P |P |        |
; |           Page-Map Level-4 Table Base Address             |      Reserved      |C |W |  Res.  | +0
; |                                                           |                    |D |T |        |
;  -----------------------------------------------------------------------------------------------
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;
; Table Base Address Field. Bits 51-12.
; This 40-bit field points to the PML4 base address.
; PML4 table is aligned on a 4-Kbyte boundary.
; The total base-address size is of 52 bits.
; (40 bits + the low-order 12 address bits (11-0) assumed to be 0).

; Page-Level Writethrough (PWT) Bit. Bit 3.
; PWT=0 => the highest-level page-translation table has a writeback caching policy.
; PWT=1 => the highest-level page-translation table has a writethrough caching policy.
;
; Page-Level Cache Disable (PCD) Bit. Bit 4.
; PCD=0 => the highest-level page-translation table is cacheable.
; PCD=1 => the highest-level page-translation table is not cacheable.
;
; Reserved Bits. Reserved fields should be cleared to 0.


; CR4
;-----
; 
;  63 62 61 60 59 58 57 56 55 54 53 52 51 50 49 48 47 46 45 44 43 42 41 40 39 38 37 36 35 34 33 32
;  -----------------------------------------------------------------------------------------------
; |                                                                                               |
; |                                     Reserved (must be zero)                                   | +4
; |                                                                                               |
;  -----------------------------------------------------------------------------------------------
; |                                                              |O |OX|P |P |M |P |P |D |T |P |V |
; |                Reserved (must be zero)                       |S |SS|C |G |C |A |S |E |S |V |M | +0
; |                                                              |X |FR|E |E |E |E |E |  |D |I |E |
;  -----------------------------------------------------------------------------------------------
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;
;
; Virtual-8086 Mode Extensions (VME) Bit. Bit 0.
; VME=1 enables hardware-support for virtual-8086 mode.
; VME=0 disables hardware-support for virtual-8086 mode.
;
; Protected-Mode Virtual Interrupts (PVI) Bit. Bit 1.
; PVI=1 enables support for protected-mode virtual interrupts.
; PVI=0 disables support for protected-mode virtual interrupts.
; (When PVI=1 hardware support of VIF and VIP bits in the rFLAGS register is enabled).
;
; Time-Stamp Disable (TSD) Bit. Bit 2.
; TSD = cleared to 0 => software at any privilege level can read the time-stamp counter (RDTSC or RDTSCP).
; TSD = set to 1 => only software at privilege-level 0 can read the time-stamp counter (RDTSC or RDTSCP).
;
; Debugging Extensions (DE) Bit. Bit 3.
; DE bit=1 enables the I/O breakpoint capability. (DR4 and DR5 reserved).
; DE bit=0 disables the I/O breakpoint capability. (DR4 and DR5 aliased to the DR6 and DR7 respectively).
;
; Page-Size Extensions (PSE) Bit. Bit 4.
; PSE=1 enables the use of 4-Mbyte physical pages.
; (With PSE=1 page-directory entry page-size field (PS) select the physical-page size between 4 Kbytes and 4 Mbytes). 
; PSE=0 disables the use of 4-Mbyte physical pages (all physical pages 4 Kbytes).
; With CR4.PAE=1 the PSE bit has no effect.(PSE bit is ignored in long mode).
;
; Physical-Address Extension (PAE) Bit. Bit 5.
; PAE=1 enables the use of physical-address extensions and 2-Mbyte physical pages.
; PAE=1 disables the use of physical-address extensions and 2-Mbyte physical pages.
; With PAE=1 the page-translation data structures are expanded from 32 to 64 bits, and the
; translation up to the 52-bit physical addresses.
; With PAE=1 the physical-page size is selectable using the page-directory-entry page-size field (PS)
; (4 Kbytes - 2 Mbytes).
; Long mode requires PAE=1.
;
; Machine-Check Enable (MCE) Bit. Bit 6.
; MCE=1 enables the machine-check exception.
; MCE=1 disables the machine-check exception.
;
; Page-Global Enable (PGE) Bit. Bit 7.
; PGE=1 enables the global-page translation.
; PGE=0 disables the global-page translation.
; When PGE is enabled you can set the global-page (G) bit in the lowest level of the page-
; translation hierarchy to 1. When the G=0 the page translation is not global.
; Page translations marked as global are not invalidated in the TLB when
; the page-translation-table base address (CR3) is updated.
;
; Performance-Monitoring Counter Enable (PCE) Bit. Bit 8.
; PCE=1 software running at any privilege level can use the RDPMC instruction.
; PCE=0 software running at most privilege level (CPL=0) can use the RDPMC instruction.
;
; FXSAVE/FXRSTOR Support (OSFXSR) Bit. Bit 9.
; OSFXSR=1 enables the use of the 128-bit media instructions. (use the FXSAVE and FXRSTOR in
; x87, 64-bit media, and 128-bit media instructions).
; OSFXSR=0 disables the use of the 128-bit media instructions.
;
; Unmasked Exception Support (OSXMMEXCPT) Bit. Bit 10.
; OSXMMEXCPT=1 supports the SIMD floating-point exception (for handling of unmasked 128-bit media
; floating-point errors).
; OSXMMEXCPT=0 disbles the support.


; EFER
;------
; 
;  63 62 61 60 59 58 57 56 55 54 53 52 51 50 49 48 47 46 45 44 43 42 41 40 39 38 37 36 35 34 33 32
;  -----------------------------------------------------------------------------------------------
; |                                                                                               |
; |                                  Reserved (must be zero)                                      | +4
; |                                                                                               |
;  -----------------------------------------------------------------------------------------------
; |                                                  |FS|  |SE|N |L |  |L |                    |S |
; |                   Reserved                       |FR|0 |V |X |M |0 |M |      Reserved      |C | +0
; |                                                  |X |  |M |E |A |  |E |                    |E |
;  -----------------------------------------------------------------------------------------------
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;
;
; System-Call Extension (SCE) Bit. Bit 0.
; SCE= 1 enables the SYSCALL and SYSRET instructions.
;
; Long Mode Enable (LME) Bit. Bit 8.
; LME=1 enables the processor to activate long mode.
; Long mode is not activated until software enables paging (some time after).
; When LME=1 and Paging is enabled => the EFER.LMA bit is set to 1 by the processor (long mode is active).
;
; Long Mode Active (LMA) Bit. Bit 10, read-only.
; This bit indicates that long mode is active.
; (LMA=1 when both long mode and paging are enabled).
; LMA=1 => compatibility mode or 64-bit mode (depending on the L bit in a code-segment descriptor).
; LMA=0 => legacy mode (standard 32-bit x86 processor).
;
; No-Execute Enable (NXE) Bit. Bit 11.
; NXE=1 => enables the no-execute page-protection
; NXE=0 => disables the no-execute page-protection
;
; Secure Virtual Machine Enable (SVME) Bit. Bit 12.
; SVME=1 => enables the SVM extensions.
; SVME=0 => disables the SVM extensions.
;
; Fast FXSAVE/FXRSTOR (FFXSR) Bit. Bit 14.
; FFXSR=1 enables the FXSAVE and FXRSTOR instructions to execute faster in 64-bit and CPL 0.
; (not saving or restoring the XMM registers XMM0-XMM15).







